/*
 * File main.cc 
 * Last modified on March 2008
 * By Yuliya Lierler
 *
 */

// Copyright 1998 by Patrik Simons
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.
//
// Patrik.Simons@hut.fi
//
//
// v0.2 Originally the version 3.76 of cmodels was taken and sup was built upon
// v3.76->v3.77 was concerned with adding Minisat1 to cmodels
// and some more options that we do not need to fix here
// v0.3 bug fixes 
// from cmodels 3.77->3.78->3.79 are ported to this version of sup
// these fixes are concerned programs with weight rules
// v0.4 bug fix (occured at CompanyControl in one place we checked the lag of a wrong atom
//1957c1957 in sup.cc
//<       if(  !curAtom->normalRuleTrueBody&& 
//INPLACE OF:
//>       if(  !atom->normalRuleTrueBody&&
//
#include <iostream>
#include <new>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "ctable.h"
#include "timer.h"
#include "interpret.h"
#include <time.h>

using namespace std;

Ctable ctable;
bool Atom::change = false;
bool Atom::conflict = false;
void a_new_handler ()
{
  cerr << "operator new failed: out of memory" << endl;
  exit(23);
}
static void timeOutHandler(int sig)
{
  switch (sig) {

  case SIGXCPU:
#ifndef SHORT_OUT
    ctable.cmodels.output.PrintStats();
#else
    ctable.cmodels.output.PrintStats();
#endif
    break;
  default:
    fprintf(stderr, "Unknown event\n");
    break;
  }

  exit (22);
}

void usage ()
{
  cerr << "Usage: sup [number] {-version]"
	// [--dlp] [-si] [-zc]  [-rs]  [-verify number] [-s] [-ext <string>] [-statistics] [name_config_file]"
	// << endl << "[--dlp] disjunctive logic program ( '|' is used for disjunction   a1 | a2 | ... | an :- body. or  { a(X) : b(X) } :- body. Choice rules are forbidden) Used when lparse is called with --dlp flag."
	// << endl << "[-zc] zchaff to call (default) "           
	//	<< endl << "[-ms] minisat to call "
	// << endl << "[-si] simo to call " 
	// << endl << "[-rs] relsat vs.2 to call"
	   << endl << "[number] number of models of completion to compute"
	   << endl << "0 - stands for all, 1 is default"	
	// << endl << "[-verify number] for disjunctive programs verifies that solution is an answer set using GNT-method 0, GNT-method-enhanced by modularity as in DLV 1(default)"
	   <<endl<< "[-version] prints out the version number"
	   << endl << "[-atomreason] build reason from some atoms of loop formula"
	   << endl << "[-temp] learns and forgets loop formulas (default keeps LF permanently)"
	//	   << endl << "[-eloop] build  loop formula from elementary loop"
	//	   << endl << "[-numlf <number>] how many clauses loop formula is allowed to be, if this number is larger then only part of loop formula is added"
	//     << endl << "[-le] in case of Simo performs backjumping and learning "
	//     << endl << "[-bj] in case of Simo performs backjumping"
	//	   << endl << "[-sb] identifies rules with same bodies; creates one auxiliary atom to represent this body"
	// << endl << "[name_config_file] name of configuration file, CONFIG is default" 

	// << endl << "[-ext <string>] starts intermediate files with  specified extension <string>"
	// << endl << "[-keep] keeps intermediate files "

	// << endl << "[-mcnorep] in case of mchaff invocation removes repetitions in the solutons "
	 
	   << endl << "[-out_f_c] solutions located into OUT* file"
           << endl << "[-statistics] print out statistics"
	//           << endl << "[-s] -wfm"
	// << endl << "[-heur number] number (integer) specifies the heuristic simo uses while doing the search. ";
	;
}

int main (int argc, char *argv[])
{



  Timer mainTimer;
  

  set_new_handler (&a_new_handler);
  bool error = false;
 



  strcpy(ctable.cmodels.param.cmodelsname, &argv[0][0]);

  strcpy(ctable.cmodels.param.config, "CONFIG");
  bool default_config=true;
  for (int c = 1; c < argc && !error; c++)
    {
      if (isdigit(argv[c][0])){
		ctable.cmodels.param.many = atoi(argv[c]);
	  }else
      if (argv[c][0] == '-')
		{
		  /*
		  if (strcmp (&argv[c][1], "rs") == 0){
			ctable.cmodels.param.sys = RELSAT;
			//			ctable.cmodels.param.loopFormula = true;
			
		  }	
		  else if (strcmp (&argv[c][1], "zc") == 0)
			ctable.cmodels.param.sys = ZCHAFF;	
		  else if (strcmp (&argv[c][1], "zca") == 0)
			ctable.cmodels.param.sys = ASSAT_ZCHAFF;	
		  else if (strcmp (&argv[c][1], "ms") == 0)
			ctable.cmodels.param.sys = MINISAT;	
		  else if (strcmp (&argv[c][1], "bc") == 0)
			ctable.cmodels.param.sys =  BCIRCUIT;	
		  else if (strcmp (&argv[c][1], "si") == 0){
			ctable.cmodels.param.sys = SIMO; 
		  }
		  else if (strcmp (&argv[c][1], "sb") == 0){
			ctable.cmodels.param.sort = true; 
		  }
		  else */
		  if (strcmp (&argv[c][1], "atomreason") == 0){
			ctable.cmodels.param.loopFormula = false; 
		  }
		  else if (strcmp (&argv[c][1], "temp") == 0){
			ctable.cmodels.param.temp = true; 
		  }
		  else if (strcmp (&argv[c][1], "version") == 0){
			cout<<"Sup Version 0.4\n";
			exit(0); 
		  }
		  /*		  else if (strcmp (&argv[c][1], "eloop") == 0){
			ctable.cmodels.param.eloop = true; 
			}*/
		  else if(strcmp (&argv[c][1], "statistics") == 0)
			ctable.cmodels.output.stat= true;
		  /*
		  else if (strcmp (&argv[c][1], "w") == 0)
			ctable.cmodels.param.cm_wfm = true;	
		  
		  else if(strcmp (&argv[c][1], "dlp") == 0 | 
				  strcmp (&argv[c][1], "-dlp") == 0 ){
			ctable.cmodels.program.disj= true;
			ctable.cmodels.program.disjProgramLparse=true;
		  }
		  else if(strcmp (&argv[c][1], "verify") == 0){
		    int method= atoi(&argv[c+1][0]);
			if(method==0)
			  ctable.cmodels.param.verifyMethod=MIN;
			if(method==1)
			  ctable.cmodels.param.verifyMethod=MINSCC;
			if(method>1|method<0)//default then
			  ctable.cmodels.param.verifyMethod=MINSCC;
			c++;
		  }
		  //	  else if(strcmp (&argv[c][1], "s") == 0)
		  //	ctable.cmodels.param.wf= false;
		  else if(strcmp (&argv[c][1], "printCycle") == 0)
			ctable.cmodels.param.printCycle= true;
		  */
		  else if(strcmp (&argv[c][1], "out_f_c") == 0){
			ctable.cmodels.output.out_f_c= true;
		  }
		  /*
		  else if(strcmp (&argv[c][1], "timings") == 0)
			ctable.cmodels.output.timings= true;
		  */
		  else if(strcmp (&argv[c][1], "silent") == 0){
		    ctable.cmodels.output.asparagus= SILENT; //program should not output res onlytime 
		    
			}
		  else if(strcmp (&argv[c][1], "asparagus") == 0){
		    ctable.cmodels.output.asparagus= ASPARAGUS; 
		  }
		  else if(strcmp (&argv[c][1], "asp2") == 0){
		    ctable.cmodels.output.asparagus= ASPCOMP2;
		  }
		  else if(strcmp (&argv[c][1], "onlytime") == 0){
		    ctable.cmodels.output.asparagus= SILENT; //program should not output res onlytime 
		    
		  }/*
		  else if(strcmp (&argv[c][1], "ext") == 0){
			ctable.cmodels.param.dir= true;
			strcpy(ctable.cmodels.param.dirName,&argv[c+1][0]);
			c++;
			}*/
		  else if(strcmp (&argv[c][1], "timeout") == 0){
			
			ctable.cmodels.param.timeout = atoi(&argv[c+1][0]);
			//strcpy(ctable.cmodels.dirName,&argv[c+1][0]);
			c++;
		  }/*
		  else if(strcmp (&argv[c][1], "heur") == 0){
			
			ctable.cmodels.param.heur = atoi(&argv[c+1][0]);
			//strcpy(ctable.cmodels.dirName,&argv[c+1][0]);
			c++;
		  }else if(strcmp (&argv[c][1], "numlf") == 0){
			
			ctable.cmodels.param.numLFClauses = atoi(&argv[c+1][0]);
			//strcpy(ctable.cmodels.dirName,&argv[c+1][0]);
			c++;
		  }else if(strcmp (&argv[c][1], "keep") == 0){
			ctable.cmodels.param.keep= true;
			
		  }	
		  else if(strcmp (&argv[c][1], "bt") == 0){
			ctable.cmodels.param.bt= true;
			ctable.cmodels.param.le= false;
		  }	
		  else if(strcmp (&argv[c][1], "bj") == 0){
			ctable.cmodels.param.bj= true;
			ctable.cmodels.param.le= false;
		  }		  
		  else if(strcmp (&argv[c][1], "le") == 0){
			ctable.cmodels.param.le= true;
		  }
		  else if(strcmp (&argv[c][1], "modes") == 0){		
			ctable.cmodels.param.bmodes= true;
		  }	
		  else if(strcmp (&argv[c][1], "shortr") == 0){		
			ctable.cmodels.param.shortr= true;
		  }	
		  else if(strcmp (&argv[c][1], "hf") == 0){		
			ctable.cmodels.param.hf= true;
		  }	*/
		  else error = true;
		}
      else{
		default_config = false;
		strcpy(ctable.cmodels.param.config, &argv[c][0]);
      }	  
      
      
    }
  //if the timeout was set then we will set the function for timeout
  if(ctable.cmodels.param.timeout!=0)    
     mainTimer.SetTimeout(ctable.cmodels.param.timeout, timeOutHandler);


  //This is not possble at the moment unless we include
  //clausification of loop formulas using many clauses
  //  if(ctable.cmodels.sys == 3 & ctable.cmodels.le)
    //    ctable.cmodels.sys =6; //here we force zchaff 
  //with incremental learning to be our choice of a sat solver
  
	
  //we find the path to cmodels and assume that config file is located there
  if(default_config){
    char path_to_config[100];
    int k=0; 
    path_to_config[k]='\0';
    int length = strlen(ctable.cmodels.param.cmodelsname);
    int l = length-1;
    while(l!=-1 && ctable.cmodels.param.cmodelsname[l]!='/'){
      l--;
    }
    if(l>=0){
      for( k= 0;k<=l;k++)
	path_to_config[k]= ctable.cmodels.param.cmodelsname[k];
      path_to_config[k]='\0';
    }
    strcat(path_to_config, "CONFIG");
    strcpy(ctable.cmodels.param.config, path_to_config);
  }
 
  if (error)
    {
      usage ();
      return 1;
    }
  if(ctable.cmodels.output.asparagus==STANDARD)
  	cout << "sup version 0.4 Reading...";
  ctable.cmodels.output.timerAll.start();
  int bad = ctable.read(stdin);  
  if(ctable.cmodels.output.asparagus==STANDARD)
	cout << "done"<<endl;       
  if (bad)
    {
      cerr << "Error in input" << endl;
      return 1;
    }
  //removes some setting that might be not fitting
  //for some specific SAT solver
  ctable.cmodels.param.finish();
  ctable.calculate();

  ctable.cmodels.output.timerAll.stop();
  ctable.cmodels.output.print();

  return 0;
}

